#!/usr/bin/perl -w
#Program name = commonContam
#Filename = commonContam.pl
#Version 1.4
#Richard J. Jacob
#Copyright (c) 2003 KBiosystems
#Date Mon Oct 27 11:55:04 2003

#Description:
#Identify the common contaminatints is a group of mass spectra at a specified ppm.
#Display a graph of the contimiants.
#Remove continmiants that occour in x/y spectra.

#Input:
#Directory containing MS
#Matching tolerance
#No. matches needed to remove mass list

#Output:
#Graph of commonly occuring masses
#Data file of commonly occuring masses
#text files of cleaned mass lists.

use strict;
use Tk;
use Win32::FileOp qw(BrowseForFolder);
use File::Find;
use File::Basename;
require Tk::ActivityBar;
use Tk::Carp qw(warningsToDialog fatalsToDialog);

my $perlversion = $^O;
if ($perlversion eq "MSWin32"){
	use Win32::FileOp qw(ShellExecute);
}

#GUI variables
my (
	$Main,
	$lbTolerance,
	$lbRemove,
	$edTolerance,
	$rbTolerance1,
	$rbTolerance2,
	$rbTolerance3,
	$edDirectory,
	$edRemove,
	$btRemove,
	$btProcess,
	$btExit,
	$btAbout,
	$btDirectory,
	$edFileName,
	$progressbar,
   );
#variables
my (
	$types,
	$title,
	$rootFolder,
	$file,
	@Filename_list,
	$currentfile,
	$toleranceValue,
	$toleranceType,
	$totalNoMasses,
	$totalNoMassesText,
	$removeValue,
	@allmasses_list,
	%massFiles,
	$templist,
	@tempfilelist,
	@masslist1,
	@masslist2,
	$mass1,
	$mass2,
	%masslist1_hits,
	%fileName_NoArray,
	$outfile,
	$outfile_count,
	$outfile_graph,
	$fileNo,
	@removemass_array,
	$base,
	$path,
	$type,
	$percent_done
   );

$toleranceValue = "100";
$toleranceType = "ppm";
$totalNoMasses = 0;
$totalNoMassesText = "/" . ($totalNoMasses +1);
$outfile = "CommonMasses";
$percent_done = 0;
$removeValue = 5;

$Main = MainWindow->new;
$Main->title("CommonContam");
$Main->geometry("258x262+8+8");

$btDirectory = $Main->Button(
                             -text => "Starting Directory", 
                             -cursor => "", 
                             -font => "Tahoma 8 normal",
							 -command => \&selectFolder
                             );

$btDirectory->place(-width => 99, 
                    -height => 25, 
                    -x => 0, 
                    -y => 16);

$edDirectory = $Main->Entry(
                            -cursor => "", 
                            -font => "Tahoma 8 normal",
							-textvariable => \$file,
                            );

$edDirectory->place(-width => 145, 
                    -height => 21, 
                    -x => 110, 
                    -y => 16);

$rbTolerance1 = $Main->Radiobutton(
							 -text => "ppm",
							 -value => "ppm",
							 -variable => \$toleranceType,
                                   );


$rbTolerance1->place(-width => 113, 
                     -height => 17, 
                     -x => 15, 
                     -y => 64);

$rbTolerance2 = $Main->Radiobutton(
								   -text => "Da",
							 -value => "Da",
							 -variable => \$toleranceType,
                                   );


$rbTolerance2->place(-width => 113, 
                     -height => 17, 
                     -x => 12, 
                     -y => 80);

$rbTolerance3 = $Main->Radiobutton(
								   -text => "mmu",
								   -value => "mmu",
								   -variable => \$toleranceType,
                                   );


$rbTolerance3->place(-width => 113, 
                     -height => 17, 
                     -x => 16, 
                     -y => 96);

$lbTolerance = $Main->Label(
                       -text => "Tolerance", 
                       -background => "#D4D0C8", 
                       -cursor => "", 
                       -font => "Tahoma 8 normal"
                       );

$lbTolerance->place(-width => 55, 
               -height => 13, 
               -x => 24, 
               -y => 48);


$edTolerance = $Main->Entry(
                            -cursor => "", 
                            -font => "Tahoma 8 normal",
							-textvariable => \$toleranceValue,
                            );


$edTolerance->place(-width => 41, 
                    -height => 21, 
                    -x => 110, 
                    -y => 80);

$btProcess = $Main->Button(
                           -text => "Process Files", 
                           -cursor => "", 
                           -font => "Tahoma 8 normal",
						   -command => \&processFiles
                           );


$btProcess->place(-width => 99, 
                  -height => 25, 
                  -x => 0, 
                  -y => 128);
$edFileName = $Main->Entry(
                           -cursor => "", 
                           -font => "Tahoma 8 normal",
						   -textvariable => \$outfile,
                           );


$edFileName->place(-width => 145, 
                   -height => 21, 
                   -x => 110, 
                   -y => 128);

$btRemove = $Main->Button(
                           -text => "Remove", 
                           -cursor => "", 
                           -font => "Tahoma 8 normal",
						   -command => \&removeContam
                           );


$btRemove->place(-width => 99, 
                  -height => 25, 
                  -x => 0, 
                  -y => 168);

$edRemove = $Main->Entry(
                         -cursor => "", 
                         -font => "Tahoma 8 normal",
						 -textvariable => \$removeValue,
                         );


$edRemove->place(-width => 49, 
                 -height => 21, 
                 -x => 110, 
                 -y => 168);

$lbRemove = $Main->Label(
                         -textvariable => \$totalNoMassesText,  
                         -background => "#D4D0C8", 
                         -cursor => "", 
                         -font => "Tahoma 8 normal"
                         );

$lbRemove->place(-width => 31, 
                 -height => 13, 
                 -x => 176, 
                 -y => 176);

$btExit = $Main->Button(
                        -text => "Exit", 
                        -cursor => "", 
                        -font => "Tahoma 8 normal",
						-command => \&quitProgram
                        );

$btExit->place(-width => 99, 
               -height => 25, 
               -x => 0, 
               -y => 208);

$btAbout = $Main->Button(
                        -text => "About", 
                        -cursor => "", 
                        -font => "Tahoma 8 normal",
						-command => \&about
                        );

$btAbout->place(-width => 99, 
               -height => 25, 
               -x => 110, 
               -y => 208);

$progressbar = $Main->ActivityBar(
        -width => 250,
        -length => 20,
		-interval=> 10,
    );
$progressbar->place(-width => 250, 
              -height => 20, 
              -x => 4, 
              -y => 240);


MainLoop;

############################ SUBS ############################



# Spin the message loop
MainLoop;

#######################
#selectFolder Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Select a folder using the windows GUI browser
#
#######################
sub selectFolder {
	$types = [['Text Files',['.txt']]];
	$title = 'Test';
	$rootFolder = "CSIDL_RECENT";
	($file) = BrowseForFolder ['test'];
	return ();
}

#######################
#processFiles Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#This sub routine reads in the mass list files, generates a big list of unique masses
#and count teh occurances of each of those masses in the text files. 
#The results saved in text files suitable for graphing with excel. 
#
#
#######################
sub processFiles {
	$progressbar->startActivity();
	$Main->update;
	#generate an array of mass spectra files
	find(\&findxml, $file);
	my $filecount = $#Filename_list;
	$filecount ++;
	my $counter = 0;
	#extract the masses from each file updating the progress bar as we go
	foreach $currentfile (@Filename_list){
		$counter ++;
		$percent_done = ($counter/$filecount)*100;
		$progressbar->configure('-value' => $percent_done);
		$Main->update;
		#open file
		my @tempMasslist;
		open (MASSLIST, "$currentfile")|| die "can't open masslist $_\n";
		while(<MASSLIST>){
			#save mass list to array
			#add mass list to all mass list
			chomp;
			push (@allmasses_list, $_);
			push (@tempMasslist, $_);
		}
		#close file
		close MASSLIST;
		#put file mass list in to hash for later
		@{$massFiles{$currentfile}} = @tempMasslist;
	}
	$progressbar->startActivity();
	$Main->update;
	#remove duplicates from allmass list
	@allmasses_list = sort {$a <=> $b}(@allmasses_list);
	(@allmasses_list) = remove_duplicates (@allmasses_list, $toleranceType, $toleranceValue);
	my @tempMasslist;
	@allmasses_list = sort {$a <=> $b}(@allmasses_list);
	@masslist1 = @allmasses_list;
	$fileNo = 0;
	@tempfilelist = sort (keys %massFiles);
	foreach $templist (@tempfilelist){
		$fileNo ++;
		$percent_done = ($fileNo/$filecount)*100;
		$progressbar->configure('-value' => $percent_done);
		$Main->update;
		$fileName_NoArray{$fileNo} = $templist;
		@masslist2 = @{$massFiles{$templist}};
		@masslist2 = sort {$a <=> $b}(@masslist2);
		foreach $mass1 (@masslist1) {
			#set massA tolerance
			my $current_tolerance = cal_mass_tolerance ($mass1, $toleranceType, $toleranceValue );
			#foreach massB in list 2
			foreach $mass2 (@masslist2){	
				if ($mass2 > $mass1 + $current_tolerance){
					#mass to high
					last;
				}
				elsif ($mass2 < $mass1 - $current_tolerance){
					#mass to low	
					next;
				}
				else {
					#mass is a match
					#count No times each mass in all mass list occures in mass lists
					#Should maybe put in mass2 to list?
					push (@{$masslist1_hits{$mass1}}, $fileNo);
				}
			}
		}
	}
	$progressbar->startActivity();
	$Main->update;
	#generate text file for excel graphing
	$outfile_count = $file . "/" . $outfile . "_count.xls";
	$outfile_graph = $file . "/" . $outfile . "_graph.xls";
	open (OUTFILECOUNT, ">$outfile_count");
	open (OUTFILEGRAPH, ">$outfile_graph");
	my @countTemp;
	#Maybe process by unprocessed allmasslist
	foreach my $tempmass (@allmasses_list){
		#print "$#{$masslist1_hits{$tempmass}}\n";
		my $tempcount = $#{$masslist1_hits{$tempmass}};
		print OUTFILECOUNT "$tempmass\t$tempcount\n";
		for (my $i= 0; $i <=$tempcount; $i++){
			print OUTFILEGRAPH "$fileName_NoArray{$masslist1_hits{$tempmass}->[$i]}\t$tempmass\t$masslist1_hits{$tempmass}->[$i]\n";
		}
		$tempcount ++;
		push (@countTemp, $tempcount);
	}
	close OUTFILECOUNT;
	close OUTFILEGRAPH; 
	@countTemp = sort {$a <=> $b} (@countTemp);
	$totalNoMasses = pop(@countTemp);
	$totalNoMassesText = "/" . ($totalNoMasses);
	$percent_done = 100;
	$progressbar->configure('-value' => $percent_done);
	$Main->update;
	return ();
}

#######################
#removeContam Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Removes all the masses that occur in x files (defined by user).
#Saves the resulting mass lists to clean text files ready for searching.
#
#######################
sub removeContam {
	#generate a list of masses to be removed
	#for each mass that occures more than $removeValue times
	my $counter =0;
	my $filecount = $#Filename_list;
	my $removeValue_corrected = $removeValue;
	$removeValue_corrected --;
	foreach my $tempmass (@allmasses_list){
		if ( $removeValue_corrected <= $#{$masslist1_hits{$tempmass}}){
			#add to remove list
			push (@removemass_array, $tempmass);
		}
	}
	#foreach file
	foreach $currentfile (@Filename_list){
		#generate a new filename
		($base,$path,$type) = fileparse ($currentfile, qr{\.txt});
		my $tempOutfile = $path. $base . "_clean.txt";
		#open the new file
		open (OUT, ">$tempOutfile");
		#foreach removemass
		@masslist1 = sort {$a <=> $b} (@removemass_array);
		@masslist2 = @{$massFiles{$currentfile}};
		@masslist2 = sort {$a <=> $b}(@masslist2);
		my %removeFromFile;
		foreach $mass1 (@masslist1) {
			#set massA tolerance
			my $current_tolerance = cal_mass_tolerance ($mass1, $toleranceType, $toleranceValue );
			#foreach massB in list 2
			foreach $mass2 (@masslist2){	
				if ($mass2 > $mass1 + $current_tolerance){
					#mass to high
					last;
				}
				elsif ($mass2 < $mass1 - $current_tolerance){
					#mass to low	
					next;
				}
				else {
					#mass is a match
					#count No times each mass in all mass list occures in mass lists
					$removeFromFile{$mass2} = 1;
				}
			}
		}
		#print printmass array to file
		foreach my $j (@{$massFiles{$currentfile}}){
			unless ($removeFromFile{$j}){
				print OUT "$j\n";
			}
		}
		
		#close file
		close OUT;
		$counter ++;
		#$percent_done = ($counter/$filecount)*100;
		$percent_done = 100;
		$progressbar->configure('-value' => $percent_done);
		$Main->update;
	}
	return ();
}


#######################
#findxml Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Uses the File::Find module to recursively locate all the txt files 
#
#######################
sub findxml {
	# get all file names that have last 3 characters "xml"
	my ( $filename_temp);
	if($File::Find::name=~/\.txt$/){
		# read in xml file
		$filename_temp = $File::Find::name;
		push (@Filename_list, $filename_temp);
		return ();
	}
	else {
		return();
	}
}

#######################
#remove_duplicates Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Removes all the duplicate masses within a defined error from a list of masses.
#Needs some refinement so that it does not work on the lowest mass but selects 
#the midle mass in a group.
#
#######################
sub remove_duplicates {
	my @variables = @_;
	my $tolerance_value = pop (@variables);
	my $tolerance_type = pop (@variables);
	my @masslist = @variables;
	my @found_duplist = ();
	my @no_duplist = ();
	my @duplist = @masslist;
	foreach my $currmass (@masslist){
		#for each current mass calculat the tolerance
		my $current_tolerance = cal_mass_tolerance ($currmass, $tolerance_type, $tolerance_value );
		#initilise duplicate list
		#set duplist2 to duplist minius the curent mass (postion 0)
		my $trash = shift (@duplist);
		my $found_dups = 0;
		#compare to privious known duplicate masses
		my $found_privious = 0;
		foreach my $item (@found_duplist){
			 if($item == $currmass){
			 	$found_privious= 1;
			 	last;
			}
		}
		unless ($found_privious ==0){
			next;
		}
		if ($currmass eq "\n"){
			push (@found_duplist, $currmass);
			next;
		}
		if ($currmass !~ /^\d+/){
			push (@found_duplist, $currmass);
			next;
		}
		foreach my $dupmass (@duplist){
			#compare to tolerance
			if ($dupmass > $currmass + $current_tolerance){
				#mass to high
				last;
			}
			elsif ($dupmass < $currmass - $current_tolerance){
				#mass to low	
				next;
			}
			else {
				#mass is a duplicate
				my $found_dups = 1;
				push (@found_duplist, $dupmass);
			}
		}
		if ($found_dups == 0){
		push (@no_duplist, $currmass);
		}
	}
	return @no_duplist;
}

#######################
#cal_mass_tolerance Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Calculates the mass error value.
#mmu not yet supported
#
#######################
sub cal_mass_tolerance {
	my $mass = $_[0];
	my $tolerance_type = $_[1];
	my $tolerance_value = $_[2];
	my $mass_tolarance = 0;
	if ($tolerance_type eq "ppm"){
		#calculate mass tolerance in ppm
		$mass_tolarance = (($tolerance_value/1e6) * $mass);
	}
	#if tolerance eq Da	
	elsif ($tolerance_type  eq "Da"){
		#use tolerance value 
		$mass_tolarance = $tolerance_value;
	}
	return $mass_tolarance;
}

#######################
#quitProgram Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Exit program
#
#######################
sub quitProgram {
		exit;
}

#######################
#about Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#Opens "About" window with www link to KBiosystems
#
#######################
sub about {
	my $Form1 = MainWindow->new;
	$Form1->title("commonContam - About");
	$Form1->geometry("450x350+8+8");
	my $Label1 = $Form1->Label(
							-text => "commonContam\n Version 1.0", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 14 bold"
							);
	$Label1->place(-width => 232, 
				   -height => 56, 
				   -x => 170, 
				   -y => 50);
	
	my $logokb = $Form1->Photo(-file=> "KBiosystemsLogo.gif");
	my $btLogokb = $Form1->Button(
                          -image => $logokb,
                          -command => sub { gologo("kb");},
                          -cursor => ""
                          );

	$btLogokb->place(-width => 59, 
                -height => 100, 
                -x => 4, 
                -y => 10);
	
	my $logoLudwig = $Form1->Photo(-file=> "LICRlogo.gif");
	my $btLogoLudwig = $Form1->Button(
                          -image => $logoLudwig,
                          -command => sub{gologo("Ludwig"),},
                          -cursor => ""
                          );

	$btLogoLudwig->place(-width => 96, 
                -height => 98, 
                -x => 4, 
                -y => 115);
	
	my $logoUCL = $Form1->Photo(-file=> "UCLlogo.gif");
	my $btLogoUCL = $Form1->Button(
                          -image => $logoUCL,
                          -command => sub{&gologo("UCL"),},
                          -cursor => ""
                          );

	$btLogoUCL->place(-width => 93, 
                -height => 111, 
                -x => 4, 
                -y => 220);

	my $Label2 = $Form1->Label(
							-text => "Processes list of masses text files from MALDI instruments\nto allow files identification and removal of contaminant\nmasses. The output is suitable for searching with Mascot or\nProteinProspector.\nBy Richard Jacob.\nCopyright (2003) KBiosysytems.\nAll rights reserved.", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 10 normal"
							);
	$Label2->place(-width => 350, 
				   -height => 200, 
				   -x => 100, 
				   -y => 115);

	my $Label3 = $Form1->Label(
							-text => "Email\: r.jacob\@ucl.ac.uk with questions and bug reports.", 
							-background => "#D4D0C8", 
							-cursor => "", 
							-font => "Tahoma 8 normal"
							);

	$Label3->place(-width => 300, 
				   -height => 30, 
				   -x => 100, 
				   -y => 300);
	return ();
}

#######################
#gologo Subroutine
#Richard Jacob
#Copyright (c) 2003 KBiosystems
#
#Description
#opens the logos  webpage
#
#######################
sub gologo {
	my ($logorequest) = @_;
	my %logohash = ("kb","http://www.kbiosystems.com/flash%20site/index.html", "Ludwig","http://www.ludwig.ucl.ac.uk/bachem_html/index.htm","UCL","http://www.biochem.ucl.ac.uk/" );
	if ($perlversion eq "MSWin32"){
		my $rc = ShellExecute("open", "$logohash{$logorequest}");
	}
	else {
		
	}
	return ();
}